//
//  CalibrationController.m
//  Skip Checker
//
//  Created by Sven on 18.10.06 for SkipChecker.
//  Copyright 2006 earthlingsoft.
//	http://earthlingsoft.net/Skip%20Checker  earthlingsoft@earthlingsoft.net
//
//	Feel free to use this code at your risk and for your own amusement. 
//	If you find it helpful and can use it for yourself, don't forget to give credit.
//	If you spot problems with the code, please let us know.

#import "CalibrationController.h"
#import "SkipChecker.h"


@implementation CalibrationController
- (id) init {
	self = [super init];
	calibrationDone = NO;
	return self;
}

/* 
when receiving showWindow, set the window to a state where calibration hasn't been done yet
*/
- (IBAction)showWindow:(id)sender {
	[self setValue:@"" forKey:CALIBRATIONTEXT];
	[self setValue:[NSNumber numberWithBool:NO] forKey:CALIBRATIONDONE];
	[super showWindow:sender];
}

/*
 do the calibration and distribute its results
 'calibrating' is nothing but reading the current SMS value and storing it
*/
- (IBAction) calibrate:(id)sender {
	[self showWindow:self];

	// info & countdown (so we don't get the click movement of the trackpad)
	[self setValue:@"3" forKey:CALIBRATIONTEXT];
	NSDictionary * errorDict;
	[[[[NSAppleScript alloc] initWithSource:@"say 3"] autorelease] executeAndReturnError:&errorDict];
	[self setValue:@"2" forKey:CALIBRATIONTEXT];
	[[[[NSAppleScript alloc] initWithSource:@"say 2"] autorelease] executeAndReturnError:&errorDict];
	[self setValue:@"1" forKey:CALIBRATIONTEXT];
	[[[[NSAppleScript alloc] initWithSource:@"say \"1.\""] autorelease] executeAndReturnError:&errorDict];

	// grab values from latest SMS reading
	SkipChecker *app = [NSApp delegate];
	NSNumber *xN = [NSNumber numberWithInt:[[app valueForKey:CURRENTXLEVEL] intValue] + [[[NSApp delegate] valueForKey:XCALIBRATION] intValue]];
	NSNumber *yN = [NSNumber numberWithInt:[[app valueForKey:CURRENTYLEVEL] intValue] + [[[NSApp delegate] valueForKey:YCALIBRATION] intValue]];
	NSNumber *zN = [NSNumber numberWithInt:[[app valueForKey:CURRENTZLEVEL] intValue] + [[[NSApp delegate] valueForKey:ZCALIBRATION] intValue]];
	
	NSDictionary *myDict = [NSDictionary dictionaryWithObjectsAndKeys:xN, xValue, yN, yValue, zN, zValue, nil];
	// NSLog ([myDict description]);
	
	// Let everybody know about the new calibration
	[app setValue:xN forKey:XCALIBRATION];
	[app setValue:yN forKey:YCALIBRATION];
	[app setValue:zN forKey:ZCALIBRATION];		
	
	// it'd be great to tie this to the built-in Ethernet MAC address (how to easily and reliably get that?)
	[DEFAULTS setValue:myDict forKey:CALIBRATIONDATA];
	[self setValue:NSLocalizedString(@"Thanks for calibrating. Enjoy.", @"Thank you string displayed after successful calibration") forKey:CALIBRATIONTEXT];
	[[[[NSAppleScript alloc] initWithSource:@"say \"Thanks.\""] autorelease] executeAndReturnError:&errorDict];
	[self setValue:[NSNumber numberWithBool:YES] forKey:CALIBRATIONDONE];
}

@end



/* 
	Transformer to give us nice text in the 'Calibrate' button.
	If we receive a NO value, calibration still needs to be done and the button should read Calibrate. If calibration has been done before and we receive the YES value, it should read Recalibrate.
*/
@implementation esCalibrateButtonTextTransformer

+ (Class)transformedValueClass;
{
    return [NSNumber class];
}

+ (BOOL)allowsReverseTransformation;
{
    return NO;   
}

- (id)transformedValue:(id)value;
{
	if ([value boolValue]) {
		return NSLocalizedString(@"Recalibrate", @"Recalibrate");
	}
	else {
		return NSLocalizedString(@"Calibrate", @"Calibrate");
	}
	}
@end



/* 
	Transformer to give us nice text in a 'Calibrate' button for an object.
	If we receive a nil value, calibration still needs to be done and the button should read 'Calibrate Motion Sensor'. If calibration has been done before and we receive a non-nil value, it should read 'Recalibrate Motion Sensor'.
*/
@implementation esCalibrateMSButtonTextTransformer

+ (Class)transformedValueClass;
{
    return [NSNumber class];
}

+ (BOOL)allowsReverseTransformation;
{
    return NO;   
}

- (id)transformedValue:(id)value;
{
	if (value) {
		return NSLocalizedString(@"Recalibrate Motion Sensor", @"Recalibrate Motion Sensor");
	}
	else {
		return NSLocalizedString(@"Calibrate Motion Sensor", @"Calibrate Motion Sensor");
	}
}
@end


