//
//  MyDocument.h
//  Skip Checker
//
//  Created by Sven on 14.09.06 for SkipChecker.
//  Copyright 2006 earthlingsoft.
//	http://earthlingsoft.net/Skip%20Checker  earthlingsoft@earthlingsoft.net
//
//	Feel free to use this code at your risk and for your own amusement. 
//	If you find it helpful and can use it for yourself, don't forget to give credit.
//	If you spot problems with the code, please let us know.

#import <Cocoa/Cocoa.h>
#import <QuartzComposer/QCView.h>
#include "Action.h"
#include "Sinewave.h"

#define XLOWERACTION @"xLowerAction"
#define XUPPERACTION @"xUpperAction"
#define YLOWERACTION @"yLowerAction"
#define YUPPERACTION @"yUpperAction"
#define ZLOWERACTION @"zLowerAction"
#define ZUPPERACTION @"zUpperAction"

#define SKIPTUNESACTION @"skipTunesAction"

#define BASEFREQUENCY1 @"baseFrequency1"
#define BASEFREQUENCY2 @"baseFrequency2"
#define FREQUENCYSHIFTFACTOR1 @"frequencyShiftFactor1"
#define FREQUENCYSHIFTFACTOR2 @"frequencyShiftFactor2"
#define ISPLAYING @"isPlaying"
#define VOLUME @"volume"
#define FREQUENCYBALANCE @"frequencyBalance"
#define VOLUMESWIVEL @"volumeSwivel"

// labels of tabs
#define WELCOMETAB @"welcome"
#define	SKIPTUNESTAB @"skiptunes"
#define ADVANCEDTAB @"advanced"
#define FUNTAB @"fun"
#define SOUNDTAB @"sound"

#define XMAX @"xMax"
#define YMAX @"yMax"
#define ZMAX @"zMax"

#define FILEFORMATVERSION @"File Format Version"

/*
 MyDocument is the document class for SkipChecker.
 Unsurprisingly it tries to handle everything document-related such as loading and saving documents.
 It will start observing the SMS levels of the application delegate and then run the appropriate action depending on the current state (which depends on the active tab and some additional flags indicating whether the Advanced tab is running and whether Sound output is active.
 The class also keeps track of the maximum SMS values we have seen so far and the maxValue/minValue of the sliders and level indicators adjust according to that - hopefully always leaving enough space for the user to make the desired settings.
*/

@interface MyDocument : NSDocument {
	float x,y,z;
	
	// limit values for sliders
	int xLimit;
	int yLimit;
	int zLimit;
	
	// maximum values we've seen so far
	int xMax;
	int yMax;
	int zMax;
	
	// Actions
	Action *xLowerAction;
	Action *xUpperAction;
	Action *yLowerAction;
	Action *yUpperAction;
	Action *zLowerAction;
	Action *zUpperAction;
	
	Action *skipTunesAction;
	
	// Sound
	IBOutlet Sinewave *wave;
	BOOL isPlaying;
	float baseFrequency1;
	float baseFrequency2;
	float frequencyShiftFactor1;
	float frequencyShiftFactor2;
	float volume;
	float frequencyBalance;
	BOOL volumeSwivel;
	
	// Vision
	IBOutlet QCView *myPatchController;

	// 
	NSNumber *contentHeight;
	NSNumber *contentWidth;
	NSNumber *repetitionInterval;
	NSString *currentTab;
	BOOL running;
}

- (void) togglePlaying:(id) sender;
- (IBAction) showCalibrationWindow:(id) sender;
- (IBAction) switchToTab:(id) sender;

@end
