//
//  MyDocument.m
//  Skip Checker
//
//  Created by Sven on 14.09.06 for SkipChecker.
//  Copyright 2006 earthlingsoft.
//	http://earthlingsoft.net/Skip%20Checker  earthlingsoft@earthlingsoft.net
//
//	Feel free to use this code at your risk and for your own amusement. 
//	If you find it helpful and can use it for yourself, don't forget to give credit.
//	If you spot problems with the code, please let us know.

#import "SkipChecker.h"
#import "MyDocument.h"

@implementation MyDocument

#pragma mark SETUP

- (id)init 
{
	// NSLog(@"[myDocument init]");
    self = [super init];
    if (self != nil) {
		NSNumber *n = [NSNumber numberWithFloat:25.0];
		NSNumber *minusN = [NSNumber numberWithFloat:-25.0];
		
		// set up actions for the Advanced tab
		[self setValue:[Action actionWithAction:@"say \"left\"" triggerLevel:minusN mode:[NSNumber numberWithInt:triggerWhenGoingBelow]] forKey:XLOWERACTION];
		[self setValue:[Action actionWithAction:@"say \"right\"" triggerLevel:n mode:[NSNumber numberWithInt:triggerWhenGoingAbove]] forKey:XUPPERACTION];
		[self setValue:[Action actionWithAction:@"say \"back\"" triggerLevel:minusN mode:[NSNumber numberWithInt:triggerWhenGoingBelow]] forKey:YLOWERACTION];
		[self setValue:[Action actionWithAction:@"say \"forth\"" triggerLevel:n mode:[NSNumber numberWithInt:triggerWhenGoingAbove]] forKey:YUPPERACTION];
		[self setValue:[Action actionWithAction:@"say \"up\"" triggerLevel:minusN mode:[NSNumber numberWithInt:triggerWhenGoingBelow]] forKey:ZLOWERACTION];
		[self setValue:[Action actionWithAction:@"say \"down\"" triggerLevel:n mode:[NSNumber numberWithInt:triggerWhenGoingAbove]] forKey:ZUPPERACTION];

		// set up action for SkipTunes
		NSDictionary *errorDict = nil;
		NSAppleScript *as = [[[NSAppleScript alloc] initWithContentsOfURL:[NSURL fileURLWithPath:[[NSBundle mainBundle]  pathForResource:@"SkipTrack" ofType:@"scpt"]] error:&errorDict] autorelease];
		if (errorDict) {
			NSLog(@"Couldn't load Skip Tunes script. Skip Tunes won't work.");
		}
		else {
			[self setValue:[Action actionWithAction:as triggerLevel:n mode:[NSNumber numberWithInt:triggerWhenGoingAbove]] forKey:SKIPTUNESACTION];
		}		
		
		// set other default values
		[self setValue:[NSNumber numberWithFloat:4.0] forKey:REPETITIONINTERVAL];
		[self setValue:WELCOMETAB forKey:CURRENTTAB];
		[self setValue:[NSNumber numberWithInt:693] forKey:CONTENTWIDTH];
		[self setValue:[NSNumber numberWithInt:468] forKey:CONTENTHEIGHT];
		
		// start observing SMS values
		SkipChecker * sC = [NSApp delegate];
		[sC addObserver:self forKeyPath:CURRENTXLEVEL options:(NSKeyValueObservingOptionNew) context:nil];
		[sC addObserver:self forKeyPath:CURRENTYLEVEL options:(NSKeyValueObservingOptionNew) context:nil];
		[sC addObserver:self forKeyPath:CURRENTZLEVEL options:(NSKeyValueObservingOptionNew) context:nil];
		
		// observe other values
		[self addObserver:self forKeyPath:BASEFREQUENCY1 options:(NSKeyValueObservingOptionNew) context:nil];
		[self addObserver:self forKeyPath:BASEFREQUENCY2 options:(NSKeyValueObservingOptionNew) context:nil];
		[self addObserver:self forKeyPath:FREQUENCYSHIFTFACTOR1 options:(NSKeyValueObservingOptionNew) context:nil];
		[self addObserver:self forKeyPath:FREQUENCYSHIFTFACTOR2 options:(NSKeyValueObservingOptionNew) context:nil];
		[self addObserver:self forKeyPath:VOLUME options:(NSKeyValueObservingOptionNew) context:nil];		
		[self addObserver:self forKeyPath:FREQUENCYBALANCE options:(NSKeyValueObservingOptionNew) context:nil];		
					
		// default values for Frequencies
		[self setValue:[NSNumber numberWithFloat:440.0] forKey:BASEFREQUENCY1];
		[self setValue:[NSNumber numberWithFloat:110.0] forKey:BASEFREQUENCY2];				
		[self setValue:[NSNumber numberWithFloat:30.0] forKey:FREQUENCYSHIFTFACTOR1];
		[self setValue:[NSNumber numberWithFloat:50.0] forKey:FREQUENCYSHIFTFACTOR2];			
		[self setValue:[NSNumber numberWithFloat:0.5] forKey:VOLUME];
	}
	
	return self;
}


- (void) awakeFromNib {	
	[wave setup];	
	[self setValue:[NSNumber numberWithFloat:0.5] forKey:FREQUENCYBALANCE]; // this value will be passed to wave, so it better be setup first
	[wave retain]; // do this so we can get the last say about when the wave gets deallocated (otherwise, when everything is handled by the nib file, it gets deallocated before its observers go away, causing a crash)
}


- (NSString *)windowNibName 
{
    return @"MyDocument";
}





#pragma mark LOAD & SAVE

/* 
	These defines are just used so we could keep the old file format which makes things look a bit ugly.
	The file format uses the KVC access keys of the old data model as keys in the dictionary. As the data model was cleaned up in v4 there are new KVC keys now.
	So the idea is to have two arrays of keys, the old ones for dealing with the file and the new ones for dealing with the data in memory.
	All that just so that existing files can still be read and I can smugly despise software authors who fail to provide compatibility with old file formats - let's hope that acutally works ...
*/ 
#define XLOWERLEVEL @"xLowerLevel"
#define XUPPERLEVEL @"xUpperLevel"
#define XLOWERACTIONTEXT @"xLowerActionText"
#define XUPPERACTIONTEXT @"xUpperActionText"
#define YLOWERLEVEL @"yLowerLevel"
#define YUPPERLEVEL @"yUpperLevel"
#define YLOWERACTIONTEXT @"yLowerActionText"
#define YUPPERACTIONTEXT @"yUpperActionText"
#define ZLOWERLEVEL @"zLowerLevel"
#define ZUPPERLEVEL @"zUpperLevel"
#define ZLOWERACTIONTEXT @"zLowerActionText"
#define ZUPPERACTIONTEXT @"zUpperActionText"
#define SKIPTUNESLEVEL @"skipTunesLevel"

#define STORAGEARRAYKEYS [NSArray arrayWithObjects:XLOWERLEVEL, XUPPERLEVEL, YLOWERLEVEL, YUPPERLEVEL, ZLOWERLEVEL, ZUPPERLEVEL, XLOWERACTIONTEXT, XUPPERACTIONTEXT, YLOWERACTIONTEXT, YUPPERACTIONTEXT, ZLOWERACTIONTEXT, ZUPPERACTIONTEXT, REPETITIONINTERVAL, CURRENTTAB, SKIPTUNESLEVEL, nil]

#define STORAGEARRAY [NSArray arrayWithObjects:[XLOWERACTION stringByAppendingFormat: @".%@", TRIGGERLEVEL], [XUPPERACTION stringByAppendingFormat: @".%@", TRIGGERLEVEL], [YLOWERACTION stringByAppendingFormat: @".%@", TRIGGERLEVEL], [YUPPERACTION stringByAppendingFormat: @".%@", TRIGGERLEVEL], [ZLOWERACTION stringByAppendingFormat: @".%@", TRIGGERLEVEL], [ZUPPERACTION stringByAppendingFormat: @".%@", TRIGGERLEVEL], [XLOWERACTION stringByAppendingFormat: @".%@", ACTIONTEXT], [XUPPERACTION stringByAppendingFormat: @".%@", ACTIONTEXT], [YLOWERACTION stringByAppendingFormat: @".%@", ACTIONTEXT], [YUPPERACTION stringByAppendingFormat: @".%@", ACTIONTEXT], [ZLOWERACTION stringByAppendingFormat: @".%@", ACTIONTEXT], [ZUPPERACTION stringByAppendingFormat: @".%@", ACTIONTEXT], REPETITIONINTERVAL, CURRENTTAB, [SKIPTUNESACTION stringByAppendingFormat: @".%@", TRIGGERLEVEL], nil]

#define STORAGEARRAYEXTRASV4 [NSArray arrayWithObjects:CONTENTHEIGHT, CONTENTWIDTH, BASEFREQUENCY1, BASEFREQUENCY2, FREQUENCYSHIFTFACTOR1, FREQUENCYSHIFTFACTOR2, VOLUME, FREQUENCYBALANCE, VOLUMESWIVEL, nil]

// load file magic
- (BOOL)readFromURL:(NSURL *)absoluteURL ofType:(NSString *)typeName error:(NSError **)outError {
	NSDictionary *myDict = [NSDictionary dictionaryWithContentsOfURL:absoluteURL];
	if (myDict) {
		NSArray *fileKeys = STORAGEARRAYKEYS;
		NSArray *KVCKeys = STORAGEARRAY;

		NSEnumerator *fileKeyEnumerator = [fileKeys objectEnumerator];
		NSEnumerator *KVCKeyEnumerator = [KVCKeys objectEnumerator];
		NSString *fileKey;
		NSString *KVCKey;
		id fileValue;
		// get values from version <= 3 files
		while ( (fileKey = [fileKeyEnumerator nextObject]) && (KVCKey = [KVCKeyEnumerator nextObject]) ) {
			fileValue = [myDict valueForKey:fileKey];
			if (fileValue) {
				[self setValue:fileValue forKeyPath:KVCKey];
			}		
		}
		
		// and get the new values from v4 format
		fileKeys = STORAGEARRAYEXTRASV4;
		fileKeyEnumerator = [fileKeys objectEnumerator];
		while ( fileKey = [fileKeyEnumerator nextObject] ) {
			fileValue = [myDict valueForKey:fileKey];
			if (fileValue) {
				[self setValue:fileValue forKeyPath:fileKey];
			}		
		}

		
		// make sure slider and level indicator limits are large enough
		int n,m,o;
		n = [[self valueForKeyPath:[XUPPERACTION stringByAppendingFormat: @".%@", TRIGGERLEVEL]] intValue] + LIMITDELTA;
		m = [[self valueForKeyPath:[XLOWERACTION stringByAppendingFormat: @".%@", TRIGGERLEVEL]] intValue] + LIMITDELTA;
		o = [[self valueForKey:XLIMIT] intValue];
		n = MAX(MAX(n,m), o);
		[self setValue:[NSNumber numberWithInt:n] forKey:XLIMIT];
		n = [[self valueForKeyPath:[YUPPERACTION stringByAppendingFormat: @".%@", TRIGGERLEVEL]] intValue] + LIMITDELTA;
		m = [[self valueForKeyPath:[YLOWERACTION stringByAppendingFormat: @".%@", TRIGGERLEVEL]] intValue] + LIMITDELTA;
		o = [[self valueForKey:YLIMIT] intValue];
		n = MAX(MAX(n,m), o);
		[self setValue:[NSNumber numberWithInt:n] forKey:YLIMIT];
		n = [[self valueForKeyPath:[ZUPPERACTION stringByAppendingFormat: @".%@", TRIGGERLEVEL]] intValue] + LIMITDELTA;
		m = [[self valueForKeyPath:[ZLOWERACTION stringByAppendingFormat: @".%@", TRIGGERLEVEL]] intValue] + LIMITDELTA;
		o = [[self valueForKey:ZLIMIT] intValue];
		n = MAX(MAX(n,m), o);
		[self setValue:[NSNumber numberWithInt:n] forKey:ZLIMIT];
		
		return YES;
	}
	else {
		return NO;
	}
}


/*
	writes the file to disk in a Dictionary containing all the relevant values
 
	v3: the file format was extended to include the window position 
	v4: the file format was extended to contain the data for the sound tab and a version number (just in case the file format will change)
*/
- (BOOL)writeToURL:(NSURL *)absoluteURL ofType:(NSString *)typeName error:(NSError **)outError {
	NSArray *fileKeys = STORAGEARRAYKEYS;
	NSArray *KVCKeys = STORAGEARRAY;
	NSMutableDictionary *myDict = [NSMutableDictionary dictionaryWithCapacity:[KVCKeys count] + 10];
	
	// collect the keys and data occurring in the 0th version of the file format
	NSEnumerator *fileKeyEnumerator = [fileKeys objectEnumerator];
	NSEnumerator *KVCKeyEnumerator = [KVCKeys objectEnumerator];
	NSString *fileKey;
	NSString *KVCKey;
	id KVCValue;
	
	while ( (KVCKey = [KVCKeyEnumerator nextObject]) && (fileKey = [fileKeyEnumerator nextObject])) {
		KVCValue = [self valueForKeyPath:KVCKey];
		if (KVCValue) {
			[myDict setObject:KVCValue forKey:fileKey];
		}
		else {
			NSLog([NSString stringWithFormat:@"Value %@ was nil.", KVCKey]);
		}
	}

	// add the keys and data added in the first version of the file format
	NSEnumerator *myEnumerator = [STORAGEARRAYEXTRASV4 objectEnumerator];
	NSString *myKey;
	id myValue;
	while (myKey = [myEnumerator nextObject]) {
		myValue = [self valueForKeyPath:myKey];
		if (myValue) {
			[myDict setObject:myValue forKey:myKey];
		}
		else {
			NSLog([NSString stringWithFormat:@"Value %@ was nil.", myKey]);
		}
	}
	
	// write a file format version in there just in case
	[myDict setObject:[NSNumber numberWithInt:1] forKey:FILEFORMATVERSION];
	BOOL success = [myDict writeToURL:absoluteURL atomically:YES];
	if (success) {
		[self updateChangeCount:NSChangeCleared];
	}
	return success;
}


/* Kick Cocoa to write HFS Type/Creator Codes
	"Because we can"
*/
- (NSDictionary *)fileAttributesToWriteToURL:(NSURL *)absoluteURL ofType:(NSString*)typeName forSaveOperation:(NSSaveOperationType)saveOperation originalContentsURL:(NSURL*)absoluteOriginalContentsURL error:(NSError **)outError 
{
    NSMutableDictionary *fileAttributes = 
	[[super fileAttributesToWriteToURL:absoluteURL ofType:typeName forSaveOperation:saveOperation originalContentsURL:absoluteOriginalContentsURL error:outError] mutableCopy];
    [fileAttributes setObject:[NSNumber numberWithUnsignedInt:HFSCREATORCODE] forKey:NSFileHFSCreatorCode];
    [fileAttributes setObject:[NSNumber numberWithUnsignedInt:HFSTYPECODE] forKey:NSFileHFSTypeCode];
    return [fileAttributes autorelease];
}


#pragma mark KVO (MAGIC)

/* 
	We are observing the X, Y and Z values of the SMS readings from the application delegate
	When they change, trigger the relevant actions depending on the currently active tab
*/
- (void) observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context {
	//	NSLog(@"MyDocument observeValueForKeyPath:%@", keyPath);
	
	NSNumber * newLevel = [change objectForKey:NSKeyValueChangeNewKey];
	int level = [newLevel intValue];
	int levelAbs = abs(level);
	int maxi = levelAbs + LIMITDELTA;
	
	/*	1 adjust limits if necessary
		2 then run action depending on the current tab
		3 store the current value, so the sound generator can easily grab it
		4 update maximum values if necessary
	*/
	if ([keyPath isEqualToString:CURRENTXLEVEL]) {
		// for the X coordinate
		if (maxi > xLimit) {
			[self setValue:[NSNumber numberWithInt:maxi] forKey:XLIMIT];
		}
		if ([currentTab isEqualToString:ADVANCEDTAB] && running) {
			[xLowerAction runForLevel:newLevel withDelta:repetitionInterval];
			[xUpperAction runForLevel:newLevel withDelta:repetitionInterval];
		}
		else if ([currentTab isEqualToString:FUNTAB]) {
			[myPatchController setValue:newLevel forInputKey:@"X_Rotation"];
		}
		x = level;
		
		if (levelAbs > xMax) {
			[self setValue:[NSNumber numberWithInt:levelAbs] forKey:XMAX];
		}
	}
	else if ([keyPath isEqualToString:CURRENTYLEVEL]) {
		// for the Y coordinate
		if (maxi > yLimit) {
			[self setValue:[NSNumber numberWithInt:maxi] forKey:YLIMIT];
		}
		if ([currentTab isEqualToString:ADVANCEDTAB] && running) {
			[yLowerAction runForLevel:newLevel withDelta:repetitionInterval];
			[yUpperAction runForLevel:newLevel withDelta:repetitionInterval];
		}
		else if ([currentTab isEqualToString:FUNTAB]) {
			[myPatchController setValue:newLevel forInputKey:@"Y_Rotation"];
		}
		y = level;

		if (levelAbs > yMax) {
			[self setValue:[NSNumber numberWithInt:levelAbs] forKey:YMAX];
		}
	}
	else if ([keyPath isEqualToString:CURRENTZLEVEL]) {
		// for the Z coordinate - this includes running Skip Tunes
		if (maxi > zLimit) {
			[self setValue:[NSNumber numberWithInt:maxi] forKey:ZLIMIT];
		}
		if ([currentTab isEqualToString:ADVANCEDTAB] && running) {
			[zLowerAction runForLevel:newLevel withDelta:repetitionInterval];
			[zUpperAction runForLevel:newLevel withDelta:repetitionInterval];
		}
		else if ([currentTab isEqualToString:SKIPTUNESTAB]) {
			[skipTunesAction runForLevel:newLevel withDelta:repetitionInterval];
		}
		else if ([currentTab isEqualToString:FUNTAB]) {
			float cubesize = 0.5 * ( 1.0 + level / (zMax * 0.8) );
			[myPatchController setValue:[NSNumber numberWithFloat:cubesize] forInputKey:@"Size"];
		}
		z = level;
		
		if (levelAbs > zMax) {
			[self setValue:[NSNumber numberWithInt:levelAbs] forKey:ZMAX];
		}		
	}
		
	/* additional handling when a sound is playing */
	if ([[wave valueForKey:SOUNDPLAYING] boolValue]) {
		if ([keyPath isEqualToString:CURRENTXLEVEL] || [keyPath isEqualToString:BASEFREQUENCY1] || [keyPath isEqualToString:FREQUENCYSHIFTFACTOR1]) {
			// adjust the first frequency
			[wave setFrequency1: MAX(baseFrequency1 + frequencyShiftFactor1 * x, 1.0)];
		}
		else if ([keyPath isEqualToString:CURRENTYLEVEL] || [keyPath isEqualToString:BASEFREQUENCY2] || [keyPath isEqualToString:FREQUENCYSHIFTFACTOR2]) {
			// adjust the second frequency
			[wave setFrequency2: MAX(baseFrequency2 + frequencyShiftFactor2 * y, 1.0)];
		}
		else if ([keyPath isEqualToString:CURRENTZLEVEL] || [keyPath isEqualToString:VOLUME]) {
			float adjustedVolume;
			if (volumeSwivel) {
				adjustedVolume = MIN(1, MAX(0, volume + (z * 0.5 / zMax)));
			}
			else {
				adjustedVolume = volume;
			}
			[wave setValue:[NSNumber numberWithFloat:adjustedVolume] forKey:WAVEVOLUME];
		}	
		
	}
	else {
		if ([keyPath isEqualToString:VOLUME]) {
			[wave setValue:newLevel forKey:WAVEVOLUME];
		}		
	}
	
	// don't miss this one either
	if ([keyPath isEqualToString:FREQUENCYBALANCE]) {
		[wave setValue:newLevel forKey:MIXINGBIAS];
	}			
}


#pragma mark UI

/*
	for accepting menu commands to switch the currently visible tab
*/
- (void) switchToTab:(id) sender {
	int tag = [sender tag];
	NSString * newTab;
	
	switch (tag) {
	case 1:	newTab = WELCOMETAB; break;
	case 2: newTab = SKIPTUNESTAB; break;
	case 3: newTab = ADVANCEDTAB; break;
	case 4: newTab = FUNTAB; break;
	case 5: newTab = SOUNDTAB; break;
	}		
		
	[self setValue:newTab forKey:CURRENTTAB];
}


/*
	the button for turning the sound on and off
*/
- (void) togglePlaying:(id) sender {
	NSString *newButtonText;
	if (isPlaying) {
		// turn sound off
		[self setValue:[NSNumber numberWithBool:NO] forKey:ISPLAYING];
		[wave stop];
		newButtonText = NSLocalizedString(@"Start Playing", @"label on sound button when the sound is off");
	}
	else {
		// turn sound on
		[self setValue:[NSNumber numberWithBool:YES] forKey:ISPLAYING];
		[wave start];
		newButtonText = NSLocalizedString(@"Stop Playing", @"label on sound button when the sound is on");
	}
	
	if ( [sender isKindOfClass:[NSButton class]] ) {
		[sender setTitle:newButtonText];
	}
}


- (IBAction) showCalibrationWindow:(id) sender {
	[[[NSApp delegate] valueForKey:CALIBRATIONCONTROLLER] showWindow:sender];
}


#pragma mark HOUSEKEEPING


- (void)windowWillClose:(NSNotification *)aNotification {
	// make sure sound stops playing when window closes
	if (isPlaying) { [self togglePlaying:self]; }
}



-(void) dealloc {
	// remove observers
	SkipChecker * sC = [NSApp delegate];
	[sC removeObserver:self forKeyPath:CURRENTXLEVEL];
	[sC removeObserver:self forKeyPath:CURRENTYLEVEL];
	[sC removeObserver:self forKeyPath:CURRENTZLEVEL];
		
	[self removeObserver:self forKeyPath:BASEFREQUENCY1];
	[self removeObserver:self forKeyPath:BASEFREQUENCY2];
	[self removeObserver:self forKeyPath:FREQUENCYSHIFTFACTOR1];
	[self removeObserver:self forKeyPath:FREQUENCYSHIFTFACTOR2];
	[self removeObserver:self forKeyPath:VOLUME];
	[self removeObserver:self forKeyPath:FREQUENCYBALANCE];	
	
	[wave release]; // we retained it in awakeFromNib
	
	[super dealloc];
}

@end
