// Derived for Skip Checker http://earthlingsoft.net/Skip%20Checker
// from the Sinewave demo application by James MacCartney
// rather use the clean original than this if you are looking for a starting point for your own programming

// Sinewave - a very simple Cocoa CoreAudio app
// by James McCartney  james@audiosynth.com  http://www.audiosynth.com
//
// Sinewave - this class implements a sine oscillator with dezippered control of frequency, pan and amplitude
//


#import "Sinewave.h"

// define a C struct from the Obj-C object so audio callback can access data
typedef struct {
    @defs(Sinewave);
} sinewavedef;

// this is the audio processing callback.
OSStatus appIOProc (AudioDeviceID  inDevice, const AudioTimeStamp*  inNow, const AudioBufferList*   inInputData, const AudioTimeStamp*  inInputTime, AudioBufferList*  outOutputData, const AudioTimeStamp* inOutputTime, 
                             void* defptr)
{    
    sinewavedef*	def = defptr; // get access to Sinewave's data
    int i;
    
    // load instance vars into registers
    double phase1 = def->phase1;
    double amp1 = def->amp1;
//    double pan1 = def->pan1;
    double freq1 = def->freq1;

    double ampz1 = def->ampz1;
//    double panz1 = def->panz1;
    double freqz1 = def->freqz1;
    
    double phase2 = def->phase2;
    double amp2 = def->amp2;
//    double pan2 = def->pan2;
    double freq2 = def->freq2;
	
    double ampz2 = def->ampz2;
//    double panz2 = def->panz2;
    double freqz2 = def->freqz2;
	
	double mixingBias = def->mixingBias;

    int numSamples = def->deviceBufferSize / def->deviceFormat.mBytesPerFrame;
    
    // assume floats for now....
    float *out = outOutputData->mBuffers[0].mData;
    
    for (i=0; i<numSamples; ++i) {
    
        float wave1 = sin(phase1) * ampz1 * mixingBias;		// generate sine wave
        phase1 = phase1 + freqz1;			// increment phase

		float wave2 = sin(phase2) * ampz2 * (1.0 - mixingBias);		// generate sine wave
        phase2 = phase2 + freqz2;			// increment phase

		float wave = (wave1 + wave2) / 2.0;
		
        // write output
        *out++ = wave * 0.5;      //(1.0-panz);		// left channel
        *out++ = wave * 0.5;      //panz;			// right channel
        
        // de-zipper controls
        // panz  = 0.001 * pan  + 0.999 * panz;
        ampz1  = 0.001 * amp1  + 0.999 * ampz1;
        freqz1 = 0.001 * freq1 + 0.999 * freqz1;
        ampz2  = 0.001 * amp2  + 0.999 * ampz2;
        freqz2 = 0.001 * freq2 + 0.999 * freqz2;
    }
    
    // save registers back to object
    def->phase1 = phase1;
    def->freqz1 = freqz1;
    def->ampz1 = ampz1;
//    def->panz1 = panz1;

	def->phase2 = phase2;
    def->freqz2 = freqz2;
    def->ampz2 = ampz2;
	//    def->panz2 = panz2;
	
    return kAudioHardwareNoError;     
}


@implementation Sinewave

- (void) setup
{
    OSStatus				err = kAudioHardwareNoError;
    UInt32				count;    
    device = kAudioDeviceUnknown;
 
    initialized = NO;
   
    // get the default output device for the HAL
    count = sizeof(device);		// it is required to pass the size of the data to be returned
    err = AudioHardwareGetProperty(kAudioHardwarePropertyDefaultOutputDevice,  &count, (void *) &device);
    if (err != kAudioHardwareNoError) {
    	fprintf(stderr, "get kAudioHardwarePropertyDefaultOutputDevice error %ld\n", err);
        return;
    }
    
    // get the buffersize that the default device uses for IO
    count = sizeof(deviceBufferSize);	// it is required to pass the size of the data to be returned
    err = AudioDeviceGetProperty(device, 0, false, kAudioDevicePropertyBufferSize, &count, &deviceBufferSize);
    if (err != kAudioHardwareNoError) {
    	fprintf(stderr, "get kAudioDevicePropertyBufferSize error %ld\n", err);
        return;
    }
    // fprintf(stderr, "deviceBufferSize = %ld\n", deviceBufferSize);
   
    // get a description of the data format used by the default device
    count = sizeof(deviceFormat);	// it is required to pass the size of the data to be returned
    err = AudioDeviceGetProperty(device, 0, false, kAudioDevicePropertyStreamFormat, &count, &deviceFormat);
    if (err != kAudioHardwareNoError) {
    	fprintf(stderr, "get kAudioDevicePropertyStreamFormat error %ld\n", err);
        return;
    }
    if (deviceFormat.mFormatID != kAudioFormatLinearPCM) {
    	fprintf(stderr, "mFormatID !=  kAudioFormatLinearPCM\n");
        return;
    }
    if (!(deviceFormat.mFormatFlags & kLinearPCMFormatFlagIsFloat)) {
    	fprintf(stderr, "Sorry, currently only works with float format....\n");
        return;
    }
    
    initialized = YES;
/*
    fprintf(stderr, "mSampleRate = %g\n", deviceFormat.mSampleRate);
    fprintf(stderr, "mFormatFlags = %08lX\n", deviceFormat.mFormatFlags);
    fprintf(stderr, "mBytesPerPacket = %ld\n", deviceFormat.mBytesPerPacket);
    fprintf(stderr, "mFramesPerPacket = %ld\n", deviceFormat.mFramesPerPacket);
    fprintf(stderr, "mChannelsPerFrame = %ld\n", deviceFormat.mChannelsPerFrame);
    fprintf(stderr, "mBytesPerFrame = %ld\n", deviceFormat.mBytesPerFrame);
    fprintf(stderr, "mBitsPerChannel = %ld\n", deviceFormat.mBitsPerChannel);
 */
}


- (void)setAmpVal:(double)val
{
    amp1 = val;
	amp2 = val;
}

/*
- (void)setFreqVal:(double)val
{
    freq = val * 2. * 3.14159265359 / deviceFormat.mSampleRate;
}
*/

- (void) setFrequency1:(double) val {
	freq1 = val * 2. * 3.14159265359 / deviceFormat.mSampleRate;
}

- (void) setFrequency2:(double) val {
	freq2 = val * 2. * 3.14159265359 / deviceFormat.mSampleRate;
}

- (double) frequency1 {
	return freq1 * deviceFormat.mSampleRate / 3.14159265359 / 2;
}
- (double) frequency2{
	return freq2 * deviceFormat.mSampleRate / 3.14159265359 / 2;
}

/* 
- (double) volume1 {
	return amp1;
}
- (void) setVolume1:(double) val {
	amp1 = val;
}
- (double) volume2 {
	return amp2;
}
- (void) setVolume2:(double) val {
	amp2 = val;
}
*/

- (void) setVolume: (NSNumber *) newVolume {
	amp1 = [newVolume doubleValue];
	amp2 = amp1;
}

- (NSNumber*) volume {
	return [NSNumber numberWithDouble:amp1];
}
	


- (void)setPanVal:(double)val
{
    pan1 = val;
	pan2 = val;
}

- (BOOL)start
{
    OSStatus		err = kAudioHardwareNoError;
   sinewavedef *def;

    if (!initialized) return false;
    if (soundPlaying) return false;
    
    // initialize phase and de-zipper filters.
    phase1 = 0.0;
	phase2 = 0.0;
    freqz1 = freq1;
	freqz2 = freq2;
    ampz1 = amp1;
    ampz2 = amp2;
    panz1 = pan1;
    panz2 = pan2;

    def = (sinewavedef *)self;
    
    err = AudioDeviceAddIOProc(device, appIOProc, (void *) def);	// setup our device with an IO proc
    if (err != kAudioHardwareNoError) return false;
    
    err = AudioDeviceStart(device, appIOProc);				// start playing sound through the device
    if (err != kAudioHardwareNoError) return false;

    soundPlaying = true;						// set the playing status global to true
    return true;
}

- (BOOL)stop
{
    OSStatus 	err = kAudioHardwareNoError;
    
    if (!initialized) return false;
    if (!soundPlaying) return false;
    
    err = AudioDeviceStop(device, appIOProc);				// stop playing sound through the device
    if (err != kAudioHardwareNoError) return false;

    err = AudioDeviceRemoveIOProc(device, appIOProc);			// remove the IO proc from the device
    if (err != kAudioHardwareNoError) return false;
    
    soundPlaying = false;						// set the playing status global to false
    return true;
}

@end
