//
//  SkipChecker.m
//  Skip Checker
//
//  Created by Sven on 14.09.06 for SkipChecker.
//  Copyright 2006 earthlingsoft.
//	http://earthlingsoft.net/Skip%20Checker  earthlingsoft@earthlingsoft.net
//
//	Feel free to use this code at your risk and for your own amusement. 
//	If you find it helpful and can use it for yourself, don't forget to give credit.
//	If you spot problems with the code, please let us know.


#import "SkipChecker.h"
#import "esNegativeTransformer.h"
#import "esFrequencyMusicTransformer.h"
#import "MyDocument.h"
#import "unimotion.h"

@implementation SkipChecker
- (id) init {
	// NSLog(@"[SkipChecker init]");
	[super init];

	// initialise own value transformers & register them with the application
	esNegativeTransformer *theTransformer;
	theTransformer = [[[esNegativeTransformer alloc] init] autorelease];
	[NSValueTransformer setValueTransformer:theTransformer forName:@"negativeTransformer"];
	theTransformer = [[[esTimesHundredTransformer alloc] init] autorelease];
	[NSValueTransformer setValueTransformer:theTransformer forName:@"timesHundredTransformer"];
	theTransformer = [[[esFrequencyMusicTransformer alloc] init] autorelease];
	[NSValueTransformer setValueTransformer:theTransformer forName:@"frequencyLogTransformer"];
	theTransformer = [[[esFrequencyNoteTransformer alloc] init] autorelease];
	[NSValueTransformer setValueTransformer:theTransformer forName:@"frequencyNoteTransformer"];
	theTransformer = [[[esCalibrateButtonTextTransformer alloc] init] autorelease];
	[NSValueTransformer setValueTransformer:theTransformer forName:@"calibrateTextTransformer"];
	theTransformer = [[[esCalibrateMSButtonTextTransformer alloc] init] autorelease];
	[NSValueTransformer setValueTransformer:theTransformer forName:@"calibrateMSTextTransformer"];

	// Do we have SMS?
	SMSType = detect_sms();
	if ( SMSType == unknown ) {
        NSLog(@"Could not detect a SMS\n");
        return nil;
    }
	
	
	// get calibration
	NSDictionary * myDict = [DEFAULTS dictionaryForKey:CALIBRATIONDATA];
	if(myDict) {
		// we have defaults
		xCalibration = [[myDict valueForKey:xValue] intValue];
		yCalibration = [[myDict valueForKey:yValue] intValue];		
		zCalibration = [[myDict valueForKey:zValue] intValue];
	} 
	else {
		// no defaults => don't correct.
		xCalibration = 0;
		yCalibration = 0;
		zCalibration = 0;
	}
		
		
	// Prepare the timer
	myTimer = [NSTimer timerWithTimeInterval:[[[NSUserDefaults standardUserDefaults] valueForKey:REPETITIONINTERVAL] floatValue] target:self selector:@selector(doMagic) userInfo:nil repeats:YES];	
	[[NSRunLoop currentRunLoop] addTimer:myTimer forMode:NSDefaultRunLoopMode];
		
	// watch the preference for the timer interval and set them up with a default value if necessary
	NSNumber *defaultRepetitionInterval = [[NSUserDefaults standardUserDefaults] objectForKey:REPETITIONINTERVAL];
	if (!defaultRepetitionInterval) {
		[[NSUserDefaults standardUserDefaults] setValue:[NSNumber numberWithFloat:0.05] forKey:REPETITIONINTERVAL];
	}
	[[NSUserDefaultsController sharedUserDefaultsController]  addObserver:self forKeyPath:[@"values." stringByAppendingString:REPETITIONINTERVAL] options:(NSKeyValueObservingOptionNew|NSKeyValueObservingOptionOld) context:nil];
	
	// hide excessive numbers by default
	id showing = [[NSUserDefaults standardUserDefaults] objectForKey:HIDENUMBERS];
	if (!showing) { 
		[[NSUserDefaults standardUserDefaults] setBool:YES forKey:HIDENUMBERS];
	}		
	
	return self;
}


/* 
The all-magic method that is called periodically by the timer. 
 It reads the SMS and sets the x,y and z Level values to those readings, taking into account the current calibration (which will be 0 if calibration hasn't been done.
*/
- (BOOL) doMagic {
	int x,y,z;
	BOOL ok =   read_sms(SMSType, &x, &y, &z) ;
	
	if (ok) {
		// we got an SMS reading
		// NSLog(@"SMS: %i, %i, %i (Calibration: %i, %i, %i)", x,y,z,xCalibration, yCalibration, zCalibration);
		NSNumber * xNumber = [NSNumber numberWithInt:x - xCalibration];
		NSNumber * yNumber = [NSNumber numberWithInt:y - yCalibration];
		NSNumber * zNumber = [NSNumber numberWithInt:z - zCalibration];
		
		[self setValue:xNumber  forKey:CURRENTXLEVEL];
		[self setValue:yNumber  forKey:CURRENTYLEVEL];
		[self setValue:zNumber  forKey:CURRENTZLEVEL];
	}
	else {
		// reading SMS failed
		NSLog(@"Couldn't read SMS data");
	}

	return ok;
}



- (void) observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context {
	// NSLog(@"[SkipChecker observeValueForKeyPath:%@", keyPath);

	if ( [keyPath isEqualToString:[@"values." stringByAppendingString:REPETITIONINTERVAL]] ) {
		// adjust our timer to the new firing speed 
		[myTimer invalidate];
		myTimer = [NSTimer timerWithTimeInterval:[[[NSUserDefaults standardUserDefaults] valueForKey:REPETITIONINTERVAL] floatValue] target:self selector:@selector(doMagic) userInfo:nil repeats:YES];	
		[[NSRunLoop currentRunLoop] addTimer:myTimer forMode:NSDefaultRunLoopMode];
	}	
}




#pragma mark ACTIONS & DATA

// for the various actions in the help menu
- (void) showHelp:(id) sender {
	int tag = [sender tag];
	switch (tag) {
	case 1: // earthlingsoft
		[WORKSPACE openURL:[NSURL URLWithString:@"http://earthlingsoft.net/"]];
		break;
	case 2: // Skip Checker
		[WORKSPACE openURL:[NSURL URLWithString:@"http://earthlingsoft.net/Skip%20Checker"]];
		break;
	case 3: // Send Mail
		[WORKSPACE openURL:[NSURL URLWithString:[NSString stringWithFormat:@"mailto:earthlingsoft%%40earthlingsoft.net?subject=Skip%%20Checker%%20%@", [self myVersionString]]]];
		break;
	case 4: // Paypal
		[WORKSPACE openURL: [NSURL URLWithString:@"https://www.paypal.com/xclick/business=earthlingsoft%40earthlingsoft.net&item_name=Skip%20Checker&no_shipping=1&cn=Comments&tax=0&currency_code=EUR&amount=3.33"]];
		break;
	case 5: // LGPL
		[WORKSPACE openFile:[[NSBundle mainBundle] pathForResource:@"LGPL" ofType:@"txt"]];
		break;
	case 6: // Version History
		[WORKSPACE openFile: [[NSBundle mainBundle] pathForResource:@"Version History" ofType:@"rtf"]];
		break;
	}
}





#pragma mark MISC

// return version string
- (NSString*) myVersionString {
	return [[NSBundle mainBundle] objectForInfoDictionaryKey:@"CFBundleShortVersionString"];
}


// get iTunes' icon, so the image well can grab it via bindings
- (NSData*) iTunesIcon {
	NSImage * im = [WORKSPACE iconForFile:[WORKSPACE absolutePathForAppBundleWithIdentifier:@"com.apple.itunes"]];
	[im setSize:NSMakeSize(128.0,128.0)];
	return [im TIFFRepresentation];
}


// the showNumbers setting
- (IBAction) toggleShowNumbers:(id) sender {
	int state = [sender state];
	if (state == NSOnState) {
		[[NSUserDefaults standardUserDefaults] setValue:[NSNumber numberWithInt:NSOffState] forKey:HIDENUMBERS];
	}
	else {
		[[NSUserDefaults standardUserDefaults] setValue:[NSNumber numberWithInt:NSOnState] forKey:HIDENUMBERS];
	}		
}


- (void) dealloc {
	[[NSUserDefaults standardUserDefaults] removeObserver:self forKeyPath:REPETITIONINTERVAL];	
	[super dealloc];
}	

@end
