//
//  esFrequencyMusicTransformer.m
//  Skip Checker
//
//
//  Created by Sven on 14.09.06 for SkipChecker.
//  Copyright 2006 earthlingsoft.
//	http://earthlingsoft.net/Skip%20Checker  earthlingsoft@earthlingsoft.net
//
//	Feel free to use this code at your risk and for your own amusement. 
//	If you find it helpful and can use it for yourself, don't forget to give credit.
//	If you spot problems with the code, please let us know.


#import "esFrequencyMusicTransformer.h"

#define BASEFREQUENCY 27.5


@implementation esFrequencyMusicTransformer

+ (Class)transformedValueClass;
{
    return [NSNumber class];
}

+ (BOOL)allowsReverseTransformation;
{
    return YES;   
}


- (id)transformedValue:(id)value;
{
    float originalValue;
    
    if (value == nil) return nil;
    
    // Attempt to get a reasonable value from the value object. 
    if ([value respondsToSelector: @selector(floatValue)]) {
		// handles NSString and NSNumber
        originalValue = [value floatValue]; 
    } else {
        [NSException raise: NSInternalInconsistencyException format: @"Value (%@) does not respond to -floatValue.", [value class]];
    }
	
	float resultValue = log2(originalValue / BASEFREQUENCY);
    return [NSNumber numberWithFloat: resultValue];
}


- (id)reverseTransformedValue:(id)value;
{
    float originalValue;
    
    if (value == nil) return nil;
    
    // Attempt to get a reasonable value from the value object. 
    if ([value respondsToSelector: @selector(floatValue)]) {
		// handles NSString and NSNumber
        originalValue = [value floatValue]; 
    } else {
        [NSException raise: NSInternalInconsistencyException format: @"Value (%@) does not respond to -floatValue.", [value class]];
    }
	
	float resultValue = BASEFREQUENCY * exp2(originalValue);
    return [NSNumber numberWithFloat: resultValue];
}

@end

/*
 This transformer takes a frequency as an input and returns the name of the corresponding musical note as a string if the frequency is reasonably close to that note.
*/
@implementation esFrequencyNoteTransformer

+ (Class)transformedValueClass;
{
    return [NSNumber class];
}

+ (BOOL)allowsReverseTransformation;
{
    return NO;   
}


- (id)transformedValue:(id)value;
{
    float originalValue;
    
    if (value == nil) return nil;
    
    // Attempt to get a reasonable value from the value object. 
    if ([value respondsToSelector: @selector(floatValue)]) {
		// handles NSString and NSNumber
        originalValue = [value floatValue]; 
    } else {
        [NSException raise: NSInternalInconsistencyException format: @"Value (%@) does not respond to -floatValue.", [value class]];
    }
	

	float x = log2(originalValue / 27.5);
	// 27.5 Hz is A0, thus x == 1 means A0, x == 2 means A1 etc. 
	
	// first determine whether we are reasonably close to a proper note:
	// there are 12 notes to an octave
	float note12 = x * 12.0;
	double a,b;
	a = modf(note12, &b);
	if (a > 0.5) b = b + 1.0;
	
	float delta = note12 - b;
	if ( delta < 0.1 && delta > -0.1) {
		// we are close to a note, determine the 
		int note12r = round(note12);
		int note = note12r % 12;
		
		// now determine the name of the note
		NSString *noteName = nil;
		switch (note) {
			case 0:
				noteName = NSLocalizedStringFromTable(@"A", @"NoteNames", @"Name of Note A");
				break;
			case 1:
				if (delta < 0) {
					noteName = NSLocalizedStringFromTable(@"A#",@"NoteNames",  @"Name of Note A sharp");
				}
				else {
					noteName = NSLocalizedStringFromTable( @"Bb",@"NoteNames",  @"Name of Note B flat");
				}
				break;
			case 2:
				noteName = NSLocalizedStringFromTable( @"B",@"NoteNames",  @"Name of Note B");
				break;
			case 3:
				noteName = NSLocalizedStringFromTable(@"C",@"NoteNames",  @"Name of Note C");
				break;
			case 4:
				if (delta < 0) {
					noteName = NSLocalizedStringFromTable( @"C#", @"NoteNames", @"Name of Note C sharp");
				}
				else {
					noteName = NSLocalizedStringFromTable(@"Db", @"NoteNames", @"Name of Note D flat");
				}
				break;
			case 5:
				noteName = NSLocalizedStringFromTable( @"D",@"NoteNames",  @"Name of Note D");
				break;
			case 6:
				if (delta < 0) {
					noteName = NSLocalizedStringFromTable(@"D#",@"NoteNames",  @"Name of Note D sharp");
				}
				else {
					noteName = NSLocalizedStringFromTable(@"Eb",@"NoteNames",  @"Name of Note E flat");
				}
				break;
			case 7:
				noteName = NSLocalizedStringFromTable(@"E",@"NoteNames",  @"Name of Note E");
				break;
			case 8:
				noteName = NSLocalizedStringFromTable(@"F",@"NoteNames",  @"Name of Note F");
				break;
			case 9:
				if (delta < 0) {
					noteName = NSLocalizedStringFromTable(@"F#",@"NoteNames",  @"Name of Note F sharp");
				}
				else {
					noteName = NSLocalizedStringFromTable(@"Gb",@"NoteNames",  @"Name of Note G flat");
				}
				break;
			case 10:
				noteName = NSLocalizedStringFromTable(@"G",@"NoteNames",  @"Name of Note G");
				break;
			case 11:
				if (delta < 0) {
					noteName = NSLocalizedStringFromTable(@"G#",@"NoteNames",  @"Name of Note G sharp");
				}
				else {
					noteName = NSLocalizedStringFromTable( @"Ab", @"NoteNames", @"Name of Note A flat");
				}
				break;
			default:
				noteName = @"?"; // well, this shouldn't happen
		}
				
		// now we could also determine the octave, but I'm happy with the note as it is
		// int octave = note12r / 12
		
		return noteName;
		
	}
	else {
		// we aren't close to a note, so return nothing
		return @"";
	}
}
@end


